﻿Type=Class
Version=2.02
@EndOfDesignText@
' PanelStack by Claudio Driussi
'
Sub Class_Globals
	Type StackItem (P As Panel, Arg As Object, Prev As Object)
	Public Stack As List			' the stack of panels
	Public CloseAll As Boolean		' flag for RemoveAll
	Private Parent As Activity		' the parent for the panels, is the Activity of the caller module
	Private Module As Object		' the caller module for event routing
	Private EventName As String		' the EventName of the panel for sub events on caller module
End Sub

Public Sub Initialize(p As Activity, m As Object)
	Stack.Initialize
	Parent = p
	Module = m
	CloseAll = False
End Sub

' Add a new panel to the stack
Public Sub Add(en As String, Arg As Object) As StackItem
	' the Arg parameter is used to pass parameters to the event sub
	' and to store data, can be useful to pass a Map.
	' from the second panel the StackItem store the reference to the
	' previous panel, this is useful for return data to the caller panel
	' the event: EventName + _Add is fired and that is the right place
	' to position the views on the panel
	' return the reference to the current StackItem
	Dim i As Int	: i = Stack.Size
	Dim si As StackItem
	si.Initialize
	Stack.Add(si)
	If i = 0 Then si.Prev = Null Else si.Prev = Stack.Get(i-1)
	si.P.Initialize("PanelEvent")
	si.P.Tag = Me
	si.Arg = Arg
	Parent.AddView(si.P, 0, 0, 100%x, 100%y)
	EventName = en
	If SubExists(Module, EventName & "_Add") Then CallSubDelayed2(Module, EventName & "_Add", si)
	Return si	
End Sub

' find a View in the passed panel serching the tag
Sub GetPanelViewByTag(P As Panel, tag As Object) As View
	' this is required because the stack virtualize the Views in the
	' panels, so to interact with them, we need a way to find them
	' the solution is to set a tag and then search with this sub
	For i = 0 To P.NumberOfViews -1
		If P.GetView(i).tag <> Null Then 
			If (P.GetView(i).tag = tag) Then Return P.GetView(i)
		End If
	Next
End Sub

' Find a View on the last panel searching the tag
Sub GetViewByTag(tag As Object) As View
	Return GetStackViewByTag(Stack.Size-1,tag)
End Sub

' Find a View in a panel of stack indexed by number
Sub GetStackViewByTag(Num As Int, tag As Object) As View
	Dim si As StackItem
	si = Stack.Get(Num)
	Return GetPanelViewByTag(si.P, tag)
End Sub

' remove the top StackItem from the stack
Public Sub RemoveItem(esc As Boolean) As Boolean
	' the parameter esc is used to tell if the user has confirmed
	' the panel or is escaped from it and fire different events.
	' returns True if the stack is not empty, the return value can
	' be used to close the activity when the user press the back button
	' and the stack is empty
	Dim e As Exception
	Dim i As Int
	Dim s As String
	Dim si As StackItem
	i = Stack.Size
	If i > 0 Then
		i = i-1
		si = Stack.Get(i)
		If esc = True Then s = "_esc" Else s = "_close"
		If SubExists(Module, EventName & s) Then CallSubDelayed2(Module, EventName & s, si)
		si.P.RemoveView
		Stack.RemoveAt(i)
		Return i > 0
	Else 
		Return False ' Exit from activity
	End If
End Sub

' close all panels in the stack
Public Sub RemoveAll
	' the CloseAll var is fired to inform the fired events whick
	' we are destroing the whole stack
	CloseAll = True
	Do While Stack.Size > 0
		RemoveItem(True)
	Loop
End Sub

' support for Activity_KeyPress event
Public Sub KeyPress(KeyCode As Int) As Boolean
	' this must be called from Activity_KeyPress to remove panels
	' from stack when the user press the back key
	If KeyCode = KeyCodes.KEYCODE_BACK Then 
		Return RemoveItem(True)
	Else 
        Return True
    End If 
End Sub

' route event click to the caller activity
Sub PanelEvent_Click
	If SubExists(Module, EventName & "_Click") Then CallSubDelayed(Module, EventName & "_Click")
End Sub

' route event long click to the caller activity
Sub PanelEvent_LongClick 
	If SubExists(Module, EventName & "_LongClick") Then 
		CallSubDelayed(Module, EventName & "_LongClick")
	Else
		PanelEvent_Click 
	End If
End Sub

' disabled because the click event is called twice
'Sub PanelEvent_Touch (Action As Int, X As Float, Y As Float) 
'	If SubExists(Module, EventName & "_Touch") Then 
'		CallSub2(Module, EventName & "_Touch", Array As Object(Action, X, Y) )
'	Else
'		PanelEvent_LongClick 
'	End If
'End Sub
